/// <reference path="jquery.d.ts" />
/// <reference path="TenantCommunication.d.ts" />
/// <reference path="knockout.d.ts" />
/// <reference path="Utilities.ts" />

class UserPreferences {
	constructor() {
		ko.track(this);
	}

	public userName = "";
	public userId = "";
	public password = ""; // not used
	public pincode = "";
	public refreshMinutes = 15;
	public savePassword = false;  // not used
}

class TenantData {
	constructor() {
		this.clear();
		ko.track(this);
	}

	public isAOBO:boolean;
	public companyInfo:CompanyInfo; // partner or client
	public domains:Domain[];
	public skus:Sku[];
	public lastEventRefreshMilliSec:number;

	public filteredUsers:UserInfo[];
	public userFilter:string;
	public recentUsers:UserInfo[]

	public filteredSRs:ServiceRequest[];
	public recentSRs:ServiceRequest[]; // not used

	public incidents:ServiceIncident[];
	public services:ServiceInfo[];

	// note: no need to clear this since it is same for all company
	public mcMessages:MCMessage[] = [];

	// todo: the following can cause bind error if clear
	public selectedIncident:ServiceIncident = null;
	public selectedUser:UserInfo = null;
	public selectedSR:ServiceRequest = null;
	public selectedMessage:MCMessage = null;


	public clear() {
		this.isAOBO = false;
		this.companyInfo = null;
		this.domains = [];
		this.skus = [];
		this.lastEventRefreshMilliSec = 0;

		this.filteredUsers = [];
		this.userFilter = "";
		this.recentUsers = [];

		this.filteredSRs = [];
		this.recentSRs = [];

		this.incidents = [];
		this.services = [];
		// this.mcMessages = [];
	}
}

class UserData {
	constructor() {
		ko.track(this);
	}

	public cookie = null;
	public cookieExpired = false;
	public lastCookieRefreshMilliSec = 0;
	public lastVisitedPage = ""; // to be retired
	public lastPrimaryPivot = "";
	public lastSecondPivot = "";
	public userRole:UserRole = UserRole.None;
	public partnerRole:PartnerRole = PartnerRole.FullAdministration; // assume user have all permission unless server return differnt
	public locale = "en-us";

	public flaggedCompanies:Company[] = [];
	public recentCompanies:Company[] = [];
	public filteredCompanies:Company[] = [];
	public companyFilter = "";
	public affectedCompanyIds:string[] = [];
	public clientIncidents:ServiceIncident[] = [];
	public selectedIncidentId = "";

	public clearClientList() {
		this.companyFilter = "";
		this.filteredCompanies.removeAll();
		this.selectedIncidentId = "";
	}
}


class Cache {
	public static key = "cache";
	private static _instance:Cache = null;
	private static _tenantData:TenantData;

	public get tenantData() {
		return Cache._tenantData;
	}

	public preferences:UserPreferences = new UserPreferences();
	public userData = new UserData();
	private _tenantDataDebug:TenantData;

	public savedBy = "";
	public version = "";
	public uiVersion = Config.uiVersion;

	// native interface
	public appVersion = "";
	public nativeSwipe = false;
	public nativeBack = false;
	public isNetworkAvailable = true;

	constructor() {
		assert(!Cache._instance);

		Cache._instance = this;
		Cache._tenantData = new TenantData();

		if (isDebug() && Config.allowPIISavedForDebug)
			this._tenantDataDebug = this.tenantData; // save pii only in debug version

		this.smoke();

		ko.track(this);

		var data:Cache = this.load();

		// validate version
		if (data) {
			if (!data.uiVersion || parseInt(data.uiVersion.split('.')[2]) < Config.minVersion) {
				console.log("out-of-date cache:" + localStorage.getItem(Cache.key));
				data = null;
			}
		}

		var p = this.preferences;

		if (data) {
			this.merge(this, data);
		}

		assert(p == this.preferences);
	}

	public static getInstance():Cache {
		if (!Cache._instance)
			Cache._instance = new Cache();

		return Cache._instance;
	}

	public merge(original, extended) {
		assert(typeof original === 'object');
		for (var key in extended) {
			if (!Array.isArray(extended[key]) && typeof extended[key] === 'object' && original[key]) {
				assert(typeof original[key] === 'object');
				this.merge(original[key], extended[key])
			}
			else
				original[key] = extended[key];
		}
		return original;
	}

	public clearUserDataAndPreferences() {
		this.userData = new UserData();
		this.preferences = new UserPreferences();
	}

	public smoke() {
		Trace.logInfo("test cache");

		if (!localStorage) {
			alert("localStorage not supported in this browser");
			return;
		}

		try {
			localStorage.setItem("test", "test");
			var value = localStorage.getItem("test");
			localStorage.removeItem("test");
		}
		catch (e) {

		}

		if (value != "test")
			alert("cache is disabled in this browser. Feature will not work");
	}

	public getPageName() {
		return window.location.pathname.substring(location.pathname.lastIndexOf("/") + 1);
	}

	// window.onunload cannot be called reliably, so do not use this to save cache
	public save():void {
		localStorage.setItem(Cache.key, this.toJson());
		Trace.logVerbose("Cache is saved");

	}

	public toJson() {
		this.userData.lastVisitedPage = this.getPageName();
		this.version = NativeHelper.getAppVersion();

		var json = JSON.stringify(this);

		return json;
	}

	// private
	private load() {
		var json = localStorage.getItem(Cache.key);
		Trace.logVerbose("Load() " + json);

		try {
			return JSON.parse(json);
		}
		catch (e) {
			console.log(e);
		}

		return null;
	}
}


//if (!window.AndroidFunction)
class NativeHelper {
	public exitApp():void {

	}

	public static isNetworkAvailable() {
		return Cache.getInstance().isNetworkAvailable;
	}

	public static isWiFiOn() {
		return true;
	}

	public static getAppVersion() {
		return Cache.getInstance().appVersion;
	}

	public static getUserLocale() {
		return "en-us"
	}
}

// Hook up with real native helper (in case of android) if it exists
if (window["DeviceNativeHelper"])
	window["NativeHelper"] = window["DeviceNativeHelper"];


function setNative(json) {
	var cache = Cache.getInstance();

	var parsed = JSON.parse(json);
	for (var name in parsed) {
		assert(name in cache);
		var value = parsed[name];
		cache[name] = value;
	}

	cache.save();
	return json;
};