/// <reference path="Utilities.ts" />
/// <reference path="jquery.d.ts" />
/// <reference path="knockout.d.ts" />
/// <reference path="knockout.es5.d.ts" />
/// <reference path="ViewModelBase.ts" />

class ClientsView extends ViewModelBase {
	public static menuName = "Clients";
	public static incidentPrefix = "Affected by ";
	public canRefresh = false;
	public filteredItems:Company[] = [];
	public showToolbar = true;

	public get searchPlaceHolder() {
		return "Search client by domain or name" ;
	}
	public get searchString() :string {return this.userData.companyFilter;}
	public set searchString(filter: string) {this.userData.companyFilter = filter;}

	constructor() {
		super();

		this.bind(); // This need to be called after fields created

		this.userData.affectedCompanyIds = [];
		this.asyncGetPartnerFlaggedList();

	}

	public onReload(){
		if (!this.userData.filteredCompanies.length && !this.searchString)
			this.asyncSearchCompanyInfo("", false);

		this.setSearchString(this.searchString);
	}

	public isPrimary(){
		return !this.userData.selectedIncidentId;
	}

	public getTitle(){
		if (this.userData.selectedIncidentId)
			return "Affected by " + this.userData.selectedIncidentId;
		else
			return super.getTitle();
	}

	public onSearchClearClick(){
		super.onSearchClearClick();
		this.userData.selectedIncidentId = "";
	}

	public getFlagClass(domainInfo:Company) {
		var isFlagged = false;

		for (var j = 0; j < this.userData.flaggedCompanies.length; j++) {
			if (this.userData.flaggedCompanies[j].TenantId == domainInfo.TenantId) {
				isFlagged = true;
				break;
			}
		}

		return  this.getIconClass(isFlagged ? "flag" : "unflag");
	}

	public getRightToolbarText() {
		return  this.userData.flaggedCompanies.length ? "Flagged" : "Recent";
	}

	public toggleToolbar(isRight) {
		this.rightToolbarSelected = isRight;
	}

	public getItems() {

		if (this.rightToolbarSelected) { // flag or recent
			var domains = this.userData.flaggedCompanies.length ? this.userData.flaggedCompanies : this.userData.recentCompanies;

			if (this.userData.selectedIncidentId) {
				var affected = this.userData.filteredCompanies.map(function(c){return c.TenantId;});
				domains = this._getAffectedTenants(domains, affected);
			}
			else
				this._joinAffectedTenants(domains, this.userData.affectedCompanyIds);
		}
		else{
			domains = this.userData.filteredCompanies;
		}

		return domains;
	}

	public search(searchString:string, blockUI) {
		if (this.hasInvalidCharacters(searchString)){
			Trace.showErrorMessage(Strings.InvalidSearchString);
			return;
		}


		this.asyncSearchCompanyInfo(searchString, blockUI);

		if (blockUI)
			this.logEvent("SearchClients");
	}


	public hasInvalidCharacters(s) {
		var invalid = "(['\"_,*+?^${}()|[]/\\])".split("");
		for (var i = 0; i < invalid.length; i++) {
			if (s.indexOf(invalid[i]) > -1)
				return true;
		}

		return false;
	}

	public clickItem(company:Company) {
		this.asyncGetCompanyInfo(company);

		this.logEvent("Client");
	}

	public asyncGetCompanyInfo(company : Company) {
		if (!this.isPartner())
			return;

		var serviceParams = {
			lastCookie: this.userData.cookie,
			tenantId: company.TenantId
		};

		var _this = this;
		var onSuccess = function (companyInfo : CompanyInfo) {

			if (companyInfo) {

				if (!_this.tenantData.companyInfo || _this.tenantData.companyInfo.ObjectId != company.TenantId) {
					_this.addToRecent(company);
					_this.tenantData.clear();
					_this.tenantData.companyInfo = companyInfo;
					_this.tenantData.isAOBO = true;
				}

				_this.goTo(HealthView);
			}
			else {
				Trace.showErrorMessage(Strings.ClientsNotFound)
			}
		}

		var onError

		this.asyncRequest(
			Config.serviceURL,
			Config.getCompanyInfoTag,
			serviceParams,
			onSuccess.bind(this), null, true);
	}

	public onAjaxError(statusCode, statusDescription, error, api, blockUI):boolean {
		if (api == Config.getCompanyInfoTag && statusCode == 401){
			Trace.showErrorMessage(Strings.ClientNotFound);
			return true;
		}
		else {
			return super.onAjaxError(statusCode, statusDescription, error, api, blockUI);
		}
	}

	public asyncSearchCompanyInfo(searchString:string, blockUI = false) {
		if (!this.isPartner())
			return;

		var serviceParams = {
			lastCookie: this.userData.cookie,
			searchString: searchString, // todo: remove
			filter: searchString,
			resultSize: Config.resultSize
		};

		var _this = this;
		var onSuccess = function (companies:Company[]) {

			if (companies && companies.length) {
				_this.searchString = searchString;
				_this.userData.selectedIncidentId = "";
				_this.userData.filteredCompanies = companies;

				if (blockUI)
					_this.rightToolbarSelected = false;
			}
			else {
				Trace.showErrorMessage(Strings.ClientsNotFound)
			}
		}

		this.asyncRequest(
			Config.serviceURL,
			Config.searchCompanyInfoTag,
			serviceParams,
			onSuccess.bind(this), null, blockUI);
	}

	public asyncGetPartnerFlaggedList() {
		if (!this.isPartner())
			return;

		var serviceParams = {
			lastCookie: this.userData.cookie,
		};

		this.asyncRequest(
			Config.serviceURL,
			Config.getPartnerFlaggedList,
			serviceParams,
			this.onFlaggedListReturned.bind(this));
	}

	// Tenants with active incident
	public asyncFindAffectedTenants(companyIds:string[]) {
		assert(companyIds && companyIds.length);

		if (!this.isPartner())
			return;

		var serviceParams = {
			lastCookie: this.userData.cookie,
			companyIds: companyIds
		};

		this.asyncRequest(
			Config.serviceURL,
			Config.findAffectedTenantsTag,
			serviceParams,
			this.onAffectedTenantsReturned.bind(this));
	}

	public addToRecent(client:Company) {
		// remove existing
		for (var j = 0; j < this.userData.recentCompanies.length; j++) {
			if (this.userData.recentCompanies[j].TenantId == client.TenantId) {
				this.userData.recentCompanies.splice(j, 1);
				break;
			}
		}

		// add to top
		client.Affected = false;
		this.userData.recentCompanies.unshift(client);
	}

	public onFlaggedListReturned(response:Company[]) {

		if (response)
			this.userData.flaggedCompanies = response; // in case user unflagged all clients, you should still set when return empty list

		if (response && response.length) {

			if (!this.userData.flaggedCompanies.length && !this.userData.filteredCompanies.length){
				// if both list are previous empty, default to flagged since it has data now
				this.rightToolbarSelected = true;
				//ViewModelBase.valueHasChanged(this, "rightToolbarSelected");
			}

			// update affected ones
			if (!this.userData.affectedCompanyIds.length || response.length != this.userData.flaggedCompanies.length) {

				var companyIds:string[] = response.map(function (domain) {return domain.TenantId});

				this.asyncFindAffectedTenants(companyIds);
			}
		}
	}

	public onAffectedTenantsReturned(response:string[]) {

		if (response) {
			this.userData.affectedCompanyIds = response;
			//ViewModelBase.valueHasChanged(this.cacheLocal, "flaggedCompanies");
			//ko.valueHasMutated(this, "cacheLocal"); // signal array not working so signal toolbar to refresh
		}
	}

	private _joinAffectedTenants(clients:Company[], affected:string[]) :void {
		for (var i = 0; i < clients.length; i++) {
			clients[i].Affected = (affected.indexOf(clients[i].TenantId) >= 0);
			ko.track(clients[i]); // without this getItems is not reevaluationed when Affected changes
		}
	}

	private _getAffectedTenants(clients:Company[], affected:string[]) : Company[] {
		var result = []
		for (var i = 0; i < clients.length; i++) {
			if  (affected.indexOf(clients[i].TenantId) >= 0){
				clients[i].Affected = false; // if all are affected then do not need to show
				result.push(clients[i]);
			}
		}
		return result;
	}

}



