// <reference path="Utilities.ts" />
/// <reference path="jquery.d.ts" />
/// <reference path="knockout.d.ts" />
/// <reference path="knockout.es5.d.ts" />
/// <reference path="ViewModelBase.ts" />

interface ServiceIncident {
	serviceName: string;
	featureName: string;
	incidentId: string;
	status: number;
	statusString: string;
	affectedTenantCount: number;
	messages: {
		MessageText:string;
		PublishedTime: string;
	}[];
}

class HealthView extends ViewModelBase {
	//public serviceStatus = [];
	public static menuName = "Health";
	public showToolbar = true;

	constructor() {
		super();
		//this.getServiceIncidents();
		this.bind(); // This need to be called after fields created
	}

	public onReload() {
		this.rightToolbarSelected = this.isPartnerNotAOBO();
	}

	public isPrimary(){
		return true;
	}

	public getLeftToolbarText() {
		if (this.isPartnerNotAOBO())
			return "Me"
		else
			return "Services"
	}

	public getRightToolbarText() {
		if (this.isPartnerNotAOBO())
			return "Clients"
		else
			return  "Incidents";
	}

	public static getAffectedCountText(affectedTenantCount) {
		var affected = "";
		if (affectedTenantCount > 0)
			affected = "Affected clients: " + affectedTenantCount;
		else if (affectedTenantCount === 0)
			affected = "Global incident"

		return affected;
	}

	public getServiceIncidents():ServiceIncident[] {
		var results = [];

		if (!this.rightToolbarSelected) {

			var services = this.tenantData.services;
			var events = this.tenantData.incidents;

			if (services && events) {
				for (var l = 0; l < services.length; l++) {
					results.push(this._joinServiceInfoWithIncident(services[l], events));
				}
			}

			return results;
		}
		else {
			if (this.isPartnerNotAOBO())
				events = this.userData.clientIncidents;
			else
				events = this.tenantData.incidents;

			results = events.sort(HealthView.compareServerity)
		}

		return results;
	}

	public getEmptyListString() {
		if (!this.rightToolbarSelected)
			return "Couldn't find any subscribed service.";
		else
			return "Couldn't find any incidents.";
	}

	public static compareServerity(a:ServiceIncident, b:ServiceIncident):number {
		var diff = a.status - b.status;
		if (diff)
			return diff;
		else
			return a.serviceName > b.serviceName ? 1 : -1;
	}

	// Event Handler
	public clickServiceIncident(incident:ServiceIncident) {
		if (incident.status == 99)
			return;

		this.tenantData.selectedIncident = incident;

		if (incident.incidentId)
			if (this.isPartnerNotAOBO())
				this.logEvent("ClientsIncident");
			else
				this.logEvent("Incident");
		else
			this.logEvent("Service");

		this.goTo(IncidentsView);
	}

	public static getStatus(status: string): number {
		switch (status.toLowerCase()) {
			case "service interruption":
				return Status.ServiceInterruption;
			case "service degradation":
				return Status.ServiceDegradation;
			case "restoring service":
				return Status.RestoringService;
			case "extended recovery":
				return Status.ExtendedRecovery;
			case "investigating":
				return Status.Investigating;
			case "service restored":
				return Status.ServiceRestored;
			case "false positive":
				return Status.FalsePositive;
			case "post-incident report published":
				return Status.PIRPublished;
			case "information unavailable":
				return Status.InformationUnavailable;
			case "service operational":
				return Status.ServiceOperational;
			default:
				return Status.Unknown;
		}
	}

	public static getStatusColor(status: Status) {
		assert(status >= 0);
		var color;
		switch (status) {
			case Status.ServiceOperational:
			case Status.FalsePositive:
			case Status.ServiceRestored:
			case Status.PIRPublished:
				color = "#00FF00" // green
				break;
			case Status.ServiceInterruption:
			case Status.ServiceDegradation:
			case Status.RestoringService:
			case Status.ExtendedRecovery:
			case Status.Investigating:
				color = "yellow";
				break;
			case Status.InformationUnavailable:
				color = "grey";
				break;
			case Status.Unknown:
			default:
				color = "white";
				break;
		}

		return "8px solid " + color;
	}

	public static toServiceIncident(event: Incident): ServiceIncident {
		// sort message by PublishedTime desc
		var messages = event.Messages.sort(function (m1, m2) {
			return ViewModelBase.getDate(m2.PublishedTime).getTime() - ViewModelBase.getDate(m1.PublishedTime).getTime();
		})

		return {
			serviceName: event.AffectedServiceHealthStatus[0].ServiceName,
			featureName: event.AffectedServiceHealthStatus[0].ServiceFeatureStatus[0].FeatureName,
			incidentId: event.Id,
			status: HealthView.getStatus(event.Status),
			statusString: event.Status,
			affectedTenantCount: event.AffectedTenantCount,
			messages: messages
		}
	}

	private _joinServiceInfoWithIncident(serviceInfo:ServiceInfo, incidents:ServiceIncident[]):ServiceIncident {
		var incident:ServiceIncident;
		for (var i = 0; i < incidents.length; i++) {
			if (incidents[i].serviceName == serviceInfo.ServiceName && (!incident || incidents[i].status < incident.status)){
				incident = incidents[i];
			}
		}

		if (incident) {
			//incident.incidentId = ""; // for incidentView to show other incident of same service
			return incident;
		}
		else
			return {
				serviceName: serviceInfo.ServiceName,
				featureName: "",
				incidentId:"",
				status: !this.tenantData.lastEventRefreshMilliSec ? 100 : 99, // default status should be unknown (color white) if we have not refreshed yet
				statusString: "",
				affectedTenantCount: -1,
				messages: null
			}
	}
}


class IncidentsView extends ViewModelBase {
	public canRefresh = false;
	public incidents : ServiceIncident[];

	constructor() {
		super();
		this.bind(); // This need to be called after fields created
	}

	public onReload() {
		this.incidents = this.getIncidents();
	}

	public singleIncident() {
		return this.incidents.length == 1;
	}

	public getTitle() {
		if (this.singleIncident())
			return  this.incidents[0].incidentId;
		else
			return super.getTitle();
	}

	public getSecondaryTitle() {
		if (this.singleIncident() && this.incidents[0].affectedTenantCount >= 0)
			return this.incidents[0].serviceName;
		else
			return ""; // pivot view will render it
	}

	public clickIncident(incident:ServiceIncident) {
		if (incident.affectedTenantCount > 0) {// not a global incident
			this.logEvent("AffectedTenants");
			this.asyncFindAffectedTenantsForPartner(incident);
		}
	}

	public asyncFindAffectedTenantsForPartner(incident:ServiceIncident) {
		var client = this.tenantData.isAOBO;
		var preCheckTenants = this.userData.flaggedCompanies.map(function (domain) {return domain.TenantId});

		var serviceParams = {
			lastCookie: this.userData.cookie,
			preCheckTenants: preCheckTenants,
			incidentId: this.tenantData.selectedIncident.incidentId
		}

		var _this = this;
		var callback = function (response:Company[]) {
			_this.userData.clearClientList();
			_this.userData.selectedIncidentId = incident.incidentId;
			_this.userData.filteredCompanies = response;
			this.goTo(ClientsView);
		}

		this.asyncRequest(
			Config.serviceURL,
			Config.findAffectedTenantsForPartnerTag,
			serviceParams,
			callback, null, true);
	}

	public getIncidentTitle(incident:ServiceIncident) {
		if (this.singleIncident())
			return this.viewAffectedText(incident.affectedTenantCount) || incident.serviceName;
		else
			return incident.incidentId;
	}

	public viewAffectedText(affectedTenantCount : number) {
		if (affectedTenantCount > 0)
			return "View " + affectedTenantCount + " affected client" + (affectedTenantCount > 1 ? "s":"");
		else if (affectedTenantCount === 0)
			return "Global incident"
		else
			return "";
	}

	public getIncidents():ServiceIncident[] {
		var result: ServiceIncident[] = [];
		var serviceName = this.tenantData.selectedIncident.serviceName;

		if (this.getInstance(HealthView).rightToolbarSelected)
			result = [ this.tenantData.selectedIncident ];
		else {
			result = this.tenantData.incidents.filter(function (incident) {
				return incident.serviceName == serviceName;
			});
		}

		// sort incidents by Status
		return result.sort(HealthView.compareServerity);
	}


}




