/// <reference path="Utilities.ts" />
/// <reference path="jquery.d.ts" />
/// <reference path="knockout.d.ts" />
/// <reference path="knockout.es5.d.ts" />
/// <reference path="ViewModelBase.ts" />

class User implements UserInfo {
	DisplayName = null;
	FirstName = null;
	LastName = null;
	UserPrincipalName = null;
	UsageLocation = null;
}

class UsersView extends ViewModelBase {
	public static menuName = "Users"; 

	public canRefresh = false;

	get searchString():string {
		return this.tenantData.userFilter;
	}

	set searchString(filter:string) {
		this.tenantData.userFilter = filter;
	}

	public searchPlaceHolder = "Search users";
	public showToolbar = true;

	constructor() {
		super();

		this.bind(); // This need to be called after fields created
	}

	public onReload() {
		if (!this.tenantData.filteredUsers.length && !this.searchString)
			this.asyncSearchUsers("");

		this.setSearchString(this.searchString);

		if (!this.tenantData.domains.length)
			this.asyncListDomains();

		if (!this.tenantData.skus.length)
			this.asyncListAccountSkus();

		if (!this.tenantData.companyInfo)
			this.asyncGetCompanyInfo();
	}

	public isPrimary(){
		return true;
	}

	public getItems() {
		return this.rightToolbarSelected ? this.tenantData.recentUsers : this.tenantData.filteredUsers;
	}

	public search(searchString, blockUI) { 
		this.asyncSearchUsers(searchString, blockUI);

		if (blockUI)
			this.logEvent("SearchUsers") 
	}

	public getRightBtn():string {
		if (this.isAddUserAllowed())
			return this.getIconClass("add");
		else
			return super.getRightBtn();
	}

	public clickRightBtn() {
		if (this.isAddUserAllowed()) {
			var user = new User();
			user.DisplayName = "";
			user.UserPrincipalName = "";
			this.tenantData.selectedUser = user;
			this.goTo(EditUserView);
			this.logEvent("AddUser");
		}
		else {
			super.clickRightBtn();
		}
	}

	public isAddUserAllowed() {
		var assignableDomains = this.tenantData.domains.filter(UsersView.isDomainAssignable);
		return assignableDomains.length && this.tenantData.skus;
	}

	public getSyncStatus(user: UserInfo): string {
		if (UserDetailsView.isFederatedUser(user, this.tenantData.domains))
			return "Federated";
		else if (!this.tenantData.companyInfo)
			return ""; // unknownyet
		else if (user.LastDirSyncTime && this.tenantData.companyInfo.DirectorySynchronizationEnabled)
			return Strings.UserStatusSynced;
		else
			return Strings.UserStatusInCloud;
	}

	public clickItem(user:UserInfo) {
		this.tenantData.selectedUser = user;
		this.addToRecent(user);
		this.goTo(UserDetailsView);
		this.logEvent("User")
	}

	public addToRecent(user:UserInfo) {
		// remove existing
		for (var i = 0; i < this.tenantData.recentUsers.length; i++) {
			if (this.tenantData.recentUsers[i].ObjectId == user.ObjectId) {
				this.tenantData.recentUsers.splice(i, 1);
				break;
			}
		}

		// add to top
		this.tenantData.recentUsers.unshift(user);
	}

	public onAjaxError(statusCode, statusDescription, error, api, blockUI):boolean {
		if (api == Config.searchUsersTag && statusCode == 401) {
			Trace.showErrorMessage(Strings.UserNotFound);
			return true;
		}
		else if (api == Config.getCompanyInfoTag && statusCode == 401) {
			this.tenantData.companyInfo = <CompanyInfo> {}; // workaround for Company type cannot be read for client version
			return true;
		}
		else {
			return super.onAjaxError(statusCode, statusDescription, error, api, blockUI);
		}
	}

	public asyncListDomains() {

		var serviceParams = {
			lastCookie: this.userData.cookie,
			tenantId: this.isAOBO() ? this.tenantData.companyInfo.ObjectId : null
		}

		var _this = this;
		var onSuccess = function (response:Domain[]) {

			if (response) {
				this.tenantData.domains = response;
			}
		}

		this.asyncRequest(
			Config.serviceURL,
			Config.ListDomainsTag,
			serviceParams,
			onSuccess);
	}

	public asyncGetCompanyInfo() {

		var serviceParams = {
			lastCookie: this.userData.cookie,
			tenantId: this.isAOBO() ? this.tenantData.companyInfo.ObjectId : null
		}

		var _this = this;
		var onSuccess = function (response:CompanyInfo) {
			this.tenantData.companyInfo = response;
		}

		this.asyncRequest(
			Config.serviceURL,
			Config.getCompanyInfoTag,
			serviceParams,
			onSuccess);
	}

	public asyncListAccountSkus() {

		var serviceParams = {
			lastCookie: this.userData.cookie,
			tenantId: this.isAOBO() ? this.tenantData.companyInfo.ObjectId : null
		}

		var _this = this;
		var onSuccess = function (response:Sku[]) {

			if (response) {
				this.tenantData.skus = response;
			}
		}

		this.asyncRequest(
			Config.serviceURL,
			Config.listAccountSkusTag,
			serviceParams,
			onSuccess);
	}

	public asyncSearchUsers(filter:string, blockUI = false) {
		filter = filter.toLowerCase();

		Trace.logVerbose("clear filtered users");
		this.tenantData.filteredUsers.length = 0;

		var serviceParams = {
			lastCookie: this.userData.cookie,
			filter: filter,
			tenantDomains: this.tenantData.isAOBO ? [this.tenantData.companyInfo.InitialDomain ] : []
		}

		var _this = this;
		var onSuccess = function (response:UserInfo[]) {

			if (response && response.length) {
				_this.tenantData.filteredUsers = response;
				_this.rightToolbarSelected = false;
				this.searchString = filter;
			}
			else {
				Trace.showErrorMessage("User not found")
			}
		}

		this.asyncRequest(
			Config.serviceURL,
			Config.searchUsersTag,
			serviceParams,
			onSuccess,
			null,
			blockUI);
	}

	public static isDomainAssignable(domain:Domain) {
		return domain.Authentication != DomainAuthenticationType.Federated
			&& domain.Status == DomainStatus.Verified
			&& !(domain.Capabilities & DomainCapabilities.SharepointDefault)
			&& domain.Name.toLowerCase().indexOf(".mail.onmicrosoft.com") == -1;
	}
}

class UserDetailsView extends ViewModelBase {
	public static viewName = "userDetails";

	public canRefresh = false;
	public tempPassword = "";
	public pageIndex = 0;
	public user:UserInfo;

	constructor() {
		super();
		this.bind();
	}

	public onReload() {
		this.user = this.tenantData.selectedUser;
		assert(this.user, "user is null");
		this.tempPassword = "";
		this.pageIndex = 0;
		ko.track(this.user);
	}

	public getRightBtn() {
		if (this.isEditAllowed())
			return this.getIconClass("edit");
		else
			return super.getRightBtn();
	}

	public clickRightBtn() {
		if (this.isEditAllowed()) {
			this.goTo(EditUserView);
			this.logEvent("EditUser");
		}
		else
			super.clickRightBtn();
	}

	public getUserType():string {
		switch (this.user.CloudExchangeRecipientDisplayType) {
			case UserType.PublicFolder:
				return "Public folder";
			case UserType.ConferenceRoomMailbox:
				return "Conference room";
			case UserType.EquipmentMailbox:
				return "Equipment";
			case UserType.None:
				return "Shared mailbox";
			case UserType.RemoteMailUser:
			case null:
			case UserType.ACLabelMailboxUser:
			case UserType.ACLabelSyncedMailboxUser:
			default:
				return this.user.IsLicensed ? "Licensed" : "Unlicensed";
		}
	}

	public isResetPasswordAllowed():boolean {
		if (!this.user.ObjectId)
			return false;
		else
			return true;

		/*
		switch (this.user.CloudExchangeRecipientDisplayType) {
			case null:
			case -2147483642:
			case UserType.ACLabelMailboxUser:
			case UserType.ACLabelSyncedMailboxUser:
				return true;
			default:
				return false;
		}*/
	}

	public isEditAllowed(): boolean {
		return Config.allowEditUser && !!this.tenantData.domains && !!this.tenantData.companyInfo && !!this.user.ObjectId;
	}

	public static isFederatedUser(user, domains) {
		if (!user.UserPrincipalName)
			return false;

		var userDomain = user.UserPrincipalName.split("@")[1];

		for (var i = 0; i < domains.length; i++) {
			if (domains[i].Name.toLowerCase() == userDomain.toLowerCase()) {
				var domain = domains[i];
				break;
			}

		}

		return domain && domain.Authentication == DomainAuthenticationType.Federated;
	}

	public getTitle() {
		return this.user.DisplayName;
	}

	public clickResetPassword() {
		if (UserDetailsView.isFederatedUser(this.user, this.tenantData.domains)) {
			Trace.showErrorMessage(Strings.FederatedDomain);
		}
		else {
			this.pageIndex = 1;
			this.logEvent("ResetPassword");
		}
	}

	public cancelResetPassword() {
		this.pageIndex = 0;
		this.logEvent("CancelResetPassword");
	}

	public onAjaxError(statusCode, statusDescription, error, api, blockUI):boolean {
		if (statusCode == 401) {
			Trace.showErrorMessage(Strings.AccessDenied);
			return true;
		}
		else if (statusCode == 400 && statusDescription.indexOf("UserConflictAuthenticationException") > -1) {
			Trace.showErrorMessage(Strings.FederatedDomain);
			return true;
		}
		else {
			return super.onAjaxError(statusCode, statusDescription, error, api, blockUI);
		}
	}

	public confirmResetPassword() {
		this.asyncResetPassword();
		this.pageIndex = 0;
		this.logEvent("ConfirmResetPassword");
	}

	public getImageUrl(name:string) {
		name = name || "blank";
		return "images/" + name + ".png";
	}

	public getName():string {
		return (this.user.FirstName || "") + " " + (this.user.LastName || "")
	}

	public getLocation():string {
		if (this.user.City || this.user.State || this.user.PostalCode)
			return this.user.City + ' ' + this.user.State + " " + this.user.PostalCode;
		else
			return "";
	}

	public asyncResetPassword() {

		var user = this.user;

		// Todo: compare guid instead
		if (user.UserPrincipalName.toLowerCase() == this.preferences.userName.toLocaleLowerCase()) {
			Trace.showErrorMessage("You can't reset your own password");
			return;
		}

		var serviceParams = {
			lastCookie: this.userData.cookie,
			upn: user.UserPrincipalName,
			password: "",
			tenantDomains: this.tenantData.isAOBO ? [this.tenantData.companyInfo.InitialDomain ] : []
		}

		var _this = this;
		var onSuccess = function (response:string) {
			_this.tempPassword = response;
		}

		this.asyncRequest(
			Config.serviceURL,
			Config.resetPasswordTag,
			serviceParams,
			onSuccess,
			null,
			true);
	}
}

class EditUserView extends ViewModelBase {
	public static viewName = "EditUserView";
	public canRefresh = false;
	public skipHistory = true;

	public allSkus:Sku[];
	public skus:Sku[]; // new skus
	public oldSkus: Sku[];
	public skusAdded:Sku[];
	public skusRemoved:Sku[];
	public assignableDomains:string[];
	public assignableSkus:Sku[];
	public domain:string;
	public usageLocation:string;
	public alias:string;
	public user:UserInfo;

	constructor() {
		super();
		this.bind();
	}

	public onReload() {
		this.user = ViewModelBase.clone(this.tenantData.selectedUser);
		this.alias = this.user.UserPrincipalName.split("@")[0];
		assert(this.tenantData.companyInfo);
		assert(this.tenantData.domains);

		this.assignableDomains = this.tenantData.domains
			.filter(UsersView.isDomainAssignable)
			.map(function (domain) {
				return domain.Name
			});

		// clone
		this.allSkus = ViewModelBase.clone(this.tenantData.skus);

		this.assignableSkus = this.allSkus.filter(function (sku) {
			ko.track(sku); // hack
			return sku.TargetClass == SkuTargetClass.User;
		})

		this.usageLocation =
			this.user.UsageLocation
			|| (this.tenantData.companyInfo ? this.tenantData.companyInfo.CountryLetterCode : "")
			|| "US";

		this.skus = [];
		this.oldSkus = [];

		
		if (this.user.Licenses) {// find existing skus
			for (var i = 0; i < this.user.Licenses.length; i++) {
				for (var j = 0; j < this.assignableSkus.length; j++) {
					if (this.assignableSkus[j].AccountSkuId == this.user.Licenses[i].AccountSkuId) {
						this.skus.push(this.assignableSkus[j]);
						this.oldSkus.push(this.assignableSkus[j]);
					}
				}
			}
		}
		else if (!this.user.ObjectId && this.assignableSkus.length == 1 && this.getAvailable(this.assignableSkus[0]) > 0) {
			// if only 1 sku available select it
			this.clickSkuRadio(this.assignableSkus[0]);
		}

		this.domain = this.user.UserPrincipalName.split("@")[1] || this.tenantData.domains[0].Name;
	}

	public getTitle() {
		if (this.user.ObjectId)
			return this.user.UserPrincipalName.split("@")[0];
		else
			return "Add User";
	}

	public getSecondaryTitle() {
		if (this.user.ObjectId)
			return "@" + this.user.UserPrincipalName.split("@")[1];
		else
			return super.getSecondaryTitle();
	}

	public getRightBtn():string {
		return this.getIconClass("save");
	}

	public clickRightBtn() {
		this.submit();
		this.logEvent("SaveUser");
	}

	public getLicenseLocation() {
		return "Licenses" + '(' + this.usageLocation + ')'
	}

	public submit() {
		if (!this.user.DisplayName)
			Trace.showErrorMessage(Strings.EmptyDisplayName);
		else if (!this.alias)
			Trace.showErrorMessage(Strings.EmptyAlias);
		else if (!this.user.ObjectId && !this.lettersAndNumbers(this.alias))
			Trace.showErrorMessage(Strings.InvalidAlias);
		else {
			this.user.UsageLocation = this.usageLocation;
			this.user.UserPrincipalName = this.alias + "@" + this.domain;
			this.user.IsLicensed = !!this.skus.length;

			if (this.user.ObjectId)
				this.asyncSetUser(this.user);
			else
				this.asyncAddUser(this.user);
		}
	}

	public lettersAndNumbers(s) {
		var exp = /^[0-9a-zA-Z]+$/;
		return s.match(exp);
	}

	public getSkuRadioClass(value:Sku) {
		var active = (this.skus.indexOf(value) > -1);

		return this.getIconClass(active ? "circleFill" : "circleEmpty");
	}

	public getSkuRadioText(value:Sku) {
		var available = this.getAvailable(value);
		if (available<0)
			available = 0;
		var name = EditUserView._skuMap[value.SkuPartNumber];
		return name + " (" + available + " available)";
	}

	public getAvailable(value:Sku):number {
		var total = value.ActiveUnits + value.WarningUnits;
		return total - value.ConsumedUnits;
	}

	public clickSkuRadio(value:Sku) {
		if (this.skus.indexOf(value) > -1) {
			this.skus.remove(value);
			value.ConsumedUnits--;
		}
		else if (this.getAvailable(value) <= 0) {
			Trace.showErrorMessage(Strings.NotEnoughLicense);
		}
		else {
			this.skus.push(value);
			value.ConsumedUnits++;
		}
	}

	public asyncAddUser(user:UserInfo) { 
		var licenses = this.skus.map(EditUserView.skuToLicense);  

		var serviceParams = {
			lastCookie: this.userData.cookie,
			tenantId: this.isAOBO() ? this.tenantData.companyInfo.ObjectId : null,
			user: this.user,
			licenses: licenses
		}

		var _this = this;
		var onSuccess = function (response:string) {
			_this.tenantData.skus = _this.allSkus; // apply sku changes
			_this.tenantData.selectedUser = _this.user; // apply user change

			// the main filter might not be all so do not add to list
			//_this.tenantData.filteredUsers.push(_this.user);  

			_this.goTo(UserDetailsView);

			var view = <UserDetailsView> _this.getInstance(UserDetailsView)
			view.tempPassword = response;
		}

		this.asyncRequest(
			Config.serviceURL,
			Config.addUserTag,
			serviceParams,
			onSuccess,
			null,
			true);
	}

	public licenseOnly() {
		return this.isDirsync() || UserDetailsView.isFederatedUser(this.user, this.tenantData.domains);
	}

	public isDirsync() {
		return this.user.LastDirSyncTime && this.tenantData.companyInfo && this.tenantData.companyInfo.DirectorySynchronizationEnabled;
	}

	public asyncSetUser(user: UserInfo) {
		var _this = this;
		var added = this.skus.filter(function(value){return _this.oldSkus.indexOf(value) == -1});
		var removed = this.oldSkus.filter(function (value) { return _this.skus.indexOf(value) == -1 });

		if (this.licenseOnly()) {
			var user = <UserInfo> new User();
			user.ObjectId = this.user.ObjectId;
			user.UsageLocation = this.user.UsageLocation;
		}
		else {
			user = this.user;
		}

		var serviceParams = {
			lastCookie: this.userData.cookie,
			tenantId: this.isAOBO() ? this.tenantData.companyInfo.ObjectId : null,
			//userId: this.user.ObjectId,
			user: user,
			addLicenses: added.map(EditUserView.skuToLicense),
			removeLicenses: removed.map(EditUserView.skuToLicense)
		}

		var _this = this;
		var onSuccess = function (response:UserInfo) {
			_this.tenantData.skus = _this.allSkus; // apply sku changes


			 // apply user change
			_this.user.Licenses = response.Licenses; 
			var index = _this.tenantData.filteredUsers.indexOf(_this.tenantData.selectedUser);
			assert(index > -1);
			_this.tenantData.filteredUsers.replace(_this.tenantData.selectedUser, _this.user);
			_this.tenantData.recentUsers.replace(_this.tenantData.selectedUser, _this.user); // todo: recent should be updated with getuser
			_this.tenantData.selectedUser = _this.user;
			_this.goBack();
		}

		this.asyncRequest(
			Config.serviceURL,
			Config.SetUserTag,
			serviceParams,
			onSuccess,
			null,
			true);
	}

	public static skuToLicense(sku: Sku){
		return {AccountName: sku.AccountName, SkuPartNumber: sku.SkuPartNumber };
	}

	public onAjaxError(statusCode, statusDescription, error, api, blockUI):boolean {
		if (statusCode == 400 && statusDescription.indexOf("UserAlreadyExistsException") > -1) {
			Trace.showErrorMessage(Strings.DuplicateUser);
			return true;
		}
		else if (statusCode == 400 && statusDescription.indexOf("InvalidLicenseConfigurationException") > -1) {
			Trace.showErrorMessage(Strings.ConflictLicense);
			return true;
		}
		else if (statusCode == 400 && blockUI) {
			Trace.showErrorMessage(Strings.InvalidInput);
		}
		else
			return super.onAjaxError(statusCode, statusDescription, error, api, blockUI);
	}

	private static _skuMap = {
		AAD_BASIC: "Azure Active Directory Basic",
		AAD_PREMIUM: "Azure Active Directory Premium",
		RIGHTSMANAGEMENT: "Azure Active Directory Rights",
		RIGHTSMANAGEMENT_GOV: "Azure Active Directory Rights",
		RIGHTSMANAGEMENT_FACULTY: "Azure Active Directory Rights for Faculty",
		RIGHTSMANAGEMENT_STUDENT: "Azure Active Directory Rights for Students",
		CLOUD_BI_STANDALONE: "Business Intelligence Plan 1",
		CLOUD_BI_WITH_OFFICE: "Business Intelligence Plan 1 with Office",
		SKYPELYNCPSTNB: "Business Voice",
		ECAL_SERVICES: "ECAL Services (EOA, EOP, DLP)",
		ECAL_SERVICES_FACULTY: "ECAL Services (EOA, EOP, DLP) for Faculty",
		ECAL_SERVICES_GOV: "ECAL Services (EOA, EOP, DLP) for Government",
		ECAL_SERVICES_STUDENT: "ECAL Services (EOA, EOP, DLP) for Student",
		ENTERPRISECLOUDSUITE: "Enterprise Cloud Suite",
		EMS: "Enterprise Mobility Suite",
		"6a467665 - 16a3-4abd-9d7f-562741e04bbb": "ENTERPRISEPACK_HIGHLANDER_TEST",
		EXCHANGESTANDARD_ALUMNI: "Exchange (Plan 1) for Alumni",
		EXCHANGESTANDARD_FACULTY: "Exchange (Plan 1) for Faculty",
		EXCHANGESTANDARD_STUDENT: "Exchange (Plan 1) for Students",
		EXCHANGEENTERPRISE_FACULTY: "Exchange (Plan 2) for Faculty",
		EXCHANGEENTERPRISE_STUDENT: "Exchange (Plan 2) for Students",
		EXCHANGEARCHIVE: "Exchange Archiving",
		EXCHANGEARCHIVE_ADDON: "Exchange Archiving",
		EXCHANGEARCHIVE_ADDON_GOV: "Exchange Archiving (EOA) for Government for Exchange Customers",
		EXCHANGEARCHIVE_GOV: "Exchange Archiving (EOA) for Government for Exchange Server",
		EXCHANGEARCHIVE_FACULTY: "Exchange Archiving (Exchange Server) for Faculty",
		EXCHANGEARCHIVE_STUDENT: "Exchange Archiving (Exchange Server) for Students",
		EXCHANGEARCHIVE_ADDON_FACULTY: "Exchange Archiving (Exchange) for Faculty",
		EXCHANGEARCHIVE_ADDON_STUDENT: "Exchange Archiving (Exchange) for Students",
		EOP_ENTERPRISE_PREMIUM_FACULTY: "Exchange Enterprise CAL Services (EOP, DLP) for Faculty",
		EOP_ENTERPRISE_PREMIUM_GOV: "Exchange Enterprise Cal Services (EOP, DLP) for Government (Private)",
		EOP_ENTERPRISE_PREMIUM_STUDENT: "Exchange Enterprise CAL Services (EOP, DLP) for Students",
		EXCHANGESTANDARD_GOV: "Exchange for Government (Plan 1G)",
		EXCHANGEENTERPRISE_GOV: "Exchange for Government (Plan 2G)",
		EXCHANGEDESKLESS: "Exchange Kiosk",
		EXCHANGEDESKLESS_GOV: "Exchange Kiosk for Government",
		EXCHANGESTANDARD: "Exchange Plan 1",
		EXCHANGEENTERPRISE: "Exchange Plan 2",
		EXCHANGETELCO: "Exchange POP",
		EOP_ENTERPRISE_FACULTY: "Exchange Protection for Faculty",
		EOP_ENTERPRISE_GOV: "Exchange Protection for Government (Private)",
		EOP_ENTERPRISE_STUDENT: "Exchange Protection for Student",
		EXCHANGE_ONLINE_WITH_ONEDRIVE_LITE: "Exchange with OneDrive for Business",
		PSTNADDNUMBERS_PILOT: "Extra Telephone Numbers for Business Voice",
		FOPE_ENTERPRISE_PREMIUM_PILOT: "ForeFront Premium Pilot",
		FOPE_ENTERPRISE_PILOT: "ForeFront Standard Pilot",
		INTUNE_A: "Intune A Direct",
		INTUNE_STORAGE: "Intune Additional Storage",
		INTUNE_D: "Intune D",
		MCOIMP_FACULTY: "Lync (Plan 1) for Faculty",
		MCOIMP_STUDENT: "Lync (Plan 1) for Students",
		MCOSTANDARD_FACULTY: "Lync (Plan 2) for Faculty",
		MCOSTANDARD_STUDENT: "Lync (Plan 2) for Students",
		MCOVOICECONF: "Lync (Plan 3)",
		MCOIMP_GOV: "Lync for Government (Plan 1G)",
		MCOSTANDARD_GOV: "Lync for Government (Plan 2G)",
		MCOVOICECONF_GOV: "Lync for Government (Plan 3G)",
		MCOINTERNAL: "Lync Internal Incubation and Corp to Cloud",
		MCOIMP: "Lync Plan 1",
		MCOSTANDARD: "Lync Plan 2",
		MCOVOICECONF_FACULTY: "Lync Plan 3 for Faculty",
		MCOVOICECONF_STUDENT: "Lync Plan 3 for Students",
		MCOVOICEESS: "Lync-to-phone 'Add On' to P1",
		INTUNE_O365_STANDALONE: "Mobile Device Management for Office 365",
		TESTOFFICESKU: "MSODS",
		OFFICE_BASIC: "Office 365 Basic",
		O365_BUSINESS: "Office 365 Business",
		O365_BUSINESS_ESSENTIALS: "Office 365 Business Essentials",
		O365_BUSINESS_PREMIUM: "Office 365 Business Premium",
		DEVELOPERPACK: "Office 365 Developer",
		DEVELOPERPACK_GOV: "Office 365 Developer",
		DEVELOPERPACK_PILOT: "Office 365 Developer Pilot",
		DEVELOPERPACK_B_PILOT: "Office 365 Developer Technical Preview",
		EDUPACK_FACULTY: "Office 365 Education for Faculty",
		EDUPACK_STUDENT: "Office 365 Education for Students",
		ENTERPRISEPACK_OSLO_PILOT: "Office 365 Enterprise E3 with Oslo Pilot",
		EOP_ENTERPRISE: "Office 365 Exchange Protection Enterprise",
		EOP_ENTERPRISE_B_PILOT: "Office 365 Exchange Protection Enterprise Technical Preview",
		EOP_ENTERPRISE_PREMIUM: "Office 365 Exchange Protection Premium",
		EOP_ENTERPRISE_PREMIUM_B_PILOT: "Office 365 Exchange Protection Premium Technical Preview",
		STANDARDWOFFPACK_IW_FACULTY: "Office 365 Faculty Individual SignUp",
		DEVPACK_B_PILOT: "Office 365 for Developer",
		STANDARDPACK_GOV: "Office 365 for Government (Plan G1)",
		STANDARDWOFFPACK_GOV: "Office 365 for Government (Plan G2)",
		ENTERPRISEPACK_GOV: "Office 365 for Government (Plan G3)",
		ENTERPRISEWITHSCAL_GOV: "Office 365 for Government (Plan G4)",
		DESKLESSPACK_GOV: "Office 365 for Government (Plan K1G)",
		DESKLESSWOFFPACK_GOV: "Office 365 for Government (Plan K2G)",
		MIDSIZEPACK: "Office 365 MidSize Business",
		MIDSIZEPACK_LEGACY: "Office 365 MidSize Business",
		MIDSIZEPACKWITHEXCHANGEARCHIVE_ADDON: "Office 365 MidSize Business with Exchange Archiving",
		STANDARDPACK_FACULTY: "Office 365 Plan A1 for Faculty",
		STANDARDPACK_STUDENT: "Office 365 Plan A1 for Students",
		STANDARDWOFFPACK_FACULTY: "Office 365 Plan A2 for Faculty",
		STANDARDWOFFPACK_STUDENT: "Office 365 Plan A2 for Students",
		ENTERPRISEPACK_EDULRG: "Office 365 Plan A3 for Faculty",
		ENTERPRISEPACK_FACULTY: "Office 365 Plan A3 for Faculty",
		ENTERPRISEPACK_STUDENT: "Office 365 Plan A3 for Students",
		ACADEMYPACK_PILOT: "Office 365 Plan A3 Pilot",
		ACADEMYPACK_B_PILOT: "Office 365 Plan A3 Technical Preview",
		ENTERPRISEWITHSCAL_FACULTY: "Office 365 Plan A4 for Faculty",
		ENTERPRISEWITHSCAL_STUDENT: "Office 365 Plan A4 for Students",
		STANDARDPACK: "Office 365 Plan E1",
		STANDARDPACKWITHEXCHANGEARCHIVE_ADDON: "Office 365 Plan E1 with Exchange Archiving",
		STANDARDWOFFPACK: "Office 365 Plan E2",
		ENTERPRISEPACK: "Office 365 Plan E3",
		ENTERPRISEPACKLRG: "Office 365 Plan E3",
		ENTERPRISEPACK_PILOT: "Office 365 Plan E3 Pilot",
		ENTERPRISEPACK_B_PILOT: "Office 365 Plan E3 Technical Preview",
		ENTERPRISEPACKWITHOUTPROPLUS: "Office 365 Plan E3 w/o ProPlus",
		ENTERPRISEWITHSCAL: "Office 365 Plan E4",
		ENTERPRISEWITHSCALWITHOUTPROPLUS: "Office 365 Plan E4 w/o ProPlus",
		ENTERPRISEWITHSCAL_VOICECONF: "Office 365 Plan E5",
		ENTERPRISEPACKWITHOUTPROPLUS_GOV: "Office 365 Plan G3 w/o ProPlus",
		ENTERPRISEWITHSCALWITHOUTPROPLUS_GOV: "Office 365 Plan G4 w/o ProPlus",
		DESKLESSPACK: "Office 365 Plan K1",
		DESKLESSPACK_YAMMER: "Office 365 Plan K1 with Yammer",
		DESKLESSWOFFPACK: "Office 365 Plan K2",
		LITEPACK: "Office 365 Plan P1",
		LITEPACK_VOICEESS: "Office 365 Plan P1 Voice",
		PROFESSIONAL_B_PILOT: "Office 365 Plan P2 Technical Preview",
		PROFESSIONAL_PILOT: "Office 365 Professional P1 Pilot",
		OFFICESUBSCRIPTION_B_PILOT: "Office 365 ProPlus Preview",
		SHAREPOINT_APP_HOSTING_ADDON: "Office 365 SharePoint – App Hosting",
		SHAREPOINT_APP_HOSTING_ADDON_FACULTY: "Office 365 SharePoint – App Hosting",
		SHAREPOINT_APP_HOSTING_ADDON_GOV: "Office 365 SharePoint – App Hosting",
		SHAREPOINT_CLOUD_BI_ADDON: "Office 365 SharePoint – Cloud BI",
		SHAREPOINT_CLOUD_BI_ADDON_GOV: "Office 365 SharePoint – Cloud BI",
		DUET_ADDON: "Office 365 SharePoint – DUET",
		DUET_ADDON_GOV: "Office 365 SharePoint – DUET",
		DUETPACK_B_PILOT: "Office 365 SharePoint – DUET 2.0 Technical Preview",
		DUET_ADDON_FACULTY: "Office 365 SharePoint – DUET for Faculty",
		DUET_ADDON_STUDENT: "Office 365 SharePoint – DUET for Students",
		LITEPACK_P2: "Office 365 Small Business Premium",
		LITEPACK_P2_B_PILOT: "Office 365 Small Business Premium Preview",
		DUETPACK_PILOT: "Office 365 SPO-DUET Pilot",
		FIS_SUITE: "Office 365 SPO-FIS Pilot",
		PROJECTPACK_PILOT: "Office 365 SPO-Project Pilot",
		OFFICE_STARTER: "Office 365 Starter",
		STANDARDWOFFPACK_IW_STUDENT: "Office 365 Student Individual SignUp",
		OFFICESUBSCRIPTION: "Office Professional Plus",
		OFFICESUBSCRIPTION_FACULTY: "Office Professional Plus for Faculty",
		OFFICESUBSCRIPTION_GOV: "Office Professional Plus for Government (1 Year)",
		OFFICESUBSCRIPTION_STUDENT: "Office Professional Plus for Students",
		WACSHAREPOINTSTD_FACULTY: "Office Web Apps (Plan 1) For Faculty",
		WACSHAREPOINTSTD_STUDENT: "Office Web Apps (Plan 1) For Students",
		WACSHAREPOINTENT_FACULTY: "Office Web Apps (Plan 2) For Faculty",
		WACSHAREPOINTENT_STUDENT: "Office Web Apps (Plan 2) For Students",
		WACSHAREPOINTSTD_GOV: "Office Web Apps for Government with SharePoint Plan 1G",
		WACSHAREPOINTENT_GOV: "Office Web Apps for Government with SharePoint Plan 2G",
		WACSHAREPOINTSTD: "Office Web Apps with SharePoint Plan 1",
		WACSHAREPOINTENT: "Office Web Apps with SharePoint Plan 2",
		ONEDRIVESTANDARD: "OneDrive for Business (Plan 1)",
		WACONEDRIVESTANDARD: "OneDrive for Business (Plan 1) with Office Web Apps",
		WACONEDRIVESTANDARD_GOV: "OneDrive for Business (Plan 1G) with Office Web Apps for Government",
		ONEDRIVESTANDARD_GOV: "OneDrive for Business for Government (Plan 1G)",
		POWER_BI_STANDARD: "Power BI",
		POWER_BI_STANDALONE: "Power BI for Office 365",
		POWER_BI_ADDON: "Power BI for Office 365 Add-On",
		POWER_BI_ADDON_FACULTY: "Power BI for Office 365 Add-On for Faculty",
		POWER_BI_ADDON_STUDENT: "Power BI for Office 365 Add-On for Students",
		POWER_BI_STANDALONE_FACULTY: "Power BI for Office 365 for Faculty",
		POWER_BI_STANDALONE_STUDENT: "Power BI for Office 365 for Students",
		POWER_BI_INDIVIDUAL_USER: "Power BI for Office 365 Individual User Trial",
		POWER_BI_PRO: "Power BI Pro",
		PROJECTESSENTIALS: "Project Essentials",
		PROJECTESSENTIALS_GOV: "Project Essentials for Government",
		PROJECTONLINE_PLAN_1: "Project Plan 1",
		PROJECTONLINE_PLAN_1_ADDON: "Project Plan 1",
		PROJECTONLINE_PLAN_1_ADDON_GOV: "Project Plan 1",
		PROJECTONLINE_PLAN_1_GOV: "Project Plan 1",
		PROJECTONLINE_PLAN_1_ADDON_STUDENT: "Project Plan 1  for Students",
		PROJECTONLINE_PLAN_1_STUDENT: "Project Plan 1  for Students",
		PROJECTONLINE_PLAN_1_ADDON_FACULTY: "Project Plan 1 for Faculty",
		PROJECTONLINE_PLAN_1_FACULTY: "Project Plan 1 for Faculty",
		PROJECTONLINE_PLAN_2: "Project Plan 2",
		PROJECTONLINE_PLAN_2_GOV: "Project Plan 2",
		PROJECTONLINE_PLAN_2_STUDENT: "Project Plan 2  for Students",
		PROJECTONLINE_PLAN_2_FACULTY: "Project Plan 2 for Faculty",
		PROJECTONLINE_B_PILOT: "Project PPM Plan 1 Technical Preview",
		PROJECTPACK_B_PILOT: "Project PPM Plan 2 Technical Preview",
		PROJECTCLIENT: "Project Pro for Office 365",
		PROJECTCLIENT_GOV: "Project Pro for Office 365",
		PROJECTCLIENT_FACULTY: "Project Pro for Office 365 for Faculty",
		PROJECTCLIENT_STUDENT: "Project Pro for Office 365 for Students",
		PROJECTCLIENT_B_PILOT: "Project Professional Technical Preview",
		CONFPSTNADDON_PILOT: "PSTN Audio Conferencing for Lync Plan 2",
		SHAREPOINTSTANDARD_FACULTY: "SharePoint (Plan 1) for Faculty",
		SHAREPOINTSTANDARD_STUDENT: "SharePoint (Plan 1) for Students",
		SHAREPOINTSTANDARD_YAMMER: "SharePoint (Plan 1) with Yammer",
		SHAREPOINTENTERPRISE_FACULTY: "SharePoint (Plan 2) for Faculty",
		SHAREPOINTENTERPRISE_STUDENT: "SharePoint (Plan 2) for Students",
		SHAREPOINTENTERPRISE_YAMMER: "SharePoint (Plan 2) with Yammer",
		SHAREPOINTSTANDARD_GOV: "SharePoint for Government (Plan 1G)",
		SHAREPOINTENTERPRISE_GOV: "SharePoint for Government (Plan 2G)",
		SHAREPOINTDESKLESS: "SharePoint Kiosk",
		SHAREPOINTEXTRANET: "SharePoint Partner Access",
		SHAREPOINTEXTRANET_GOV: "SharePoint Partner Access",
		SHAREPOINTEXTRANET_ALUMNI: "SharePoint Partner Access for Alumni",
		SHAREPOINTEXTRANET_FACULTY: "SharePoint Partner Access for EDU",
		SHAREPOINTEXTRANET_PARENTS: "SharePoint Partner Access for Parents",
		SHAREPOINTSTANDARD: "SharePoint Plan 1",
		SHAREPOINTENTERPRISE: "SharePoint Plan 2",
		SHAREPOINTSTORAGE: "SharePoint Storage",
		SHAREPOINTSTORAGE_FACULTY: "SharePoint Storage",
		SHAREPOINTSTORAGE_GOV: "SharePoint Storage",
		SHAREPOINTSTORAGELITE: "SharePoint Storage for OfficeLite",
		SHAREPOINTWACADDON: "SharePoint WAC Edit",
		SMB_BUSINESS: "SMB Business",
		SMB_BUSINESS_ESSENTIALS: "SMB Business Essentials",
		SMB_BUSINESS_PREMIUM: "SMB Business Premium",
		PSTNTOLLNUMBERS_PILOT: "Toll Free Telephone Numbers for Businss Voice",
		VISIOCLIENT: "Visio Pro for Office 365",
		VISIOCLIENT_GOV: "Visio Pro for Office 365",
		VISIOCLIENT_FACULTY: "Visio Pro for Office 365 for Faculty",
		VISIOCLIENT_STUDENT: "Visio Pro for Office 365 for Students",
		VISIOCLIENT_B_PILOT: "Visio Professional Technical Preview",
		WIFIENTERPRISE: "WiFi",
		YAMMER_ENTERPRISE_STANDALONE: "Yammer Enterprise Standalone"
	}

}






